#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>

#define ADC_AREF_VOLTAGE 3.3
#define PA0_adc 0
#define WATER_LEVEL_LOW 250

void ADC_init(void)
{
    // enable ADC with:
    // * reference AVCC with external capacitor at AREF pin
    // * without left adjust of conversion result
    // * no auto-trigger
    // * no interrupt
    // * prescaler at 32
    ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);

	DDRA &= ~(1 << PA0);
}

uint16_t ADC_get(uint8_t channel)
{
    // start ADC conversion on "channel"
    // wait for completion
    // return the result
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

double ADC_voltage(int raw)
{
    (void)raw;
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

void init_pump_pins()
{
	DDRB |= (1 << PB5);
	DDRB |= (1 << PB6);

	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);
	
	PORTB &= ~( 1 << PB3);
	PORTB &= ~( 1 << PB4);

}
int hole_is_open = 0;
void open_hole()
{	
		PORTB |= ( 1 << PB3);
		PORTB &= ~( 1 << PB4);
		_delay_ms(600);
		PORTB &= ~( 1 << PB3);
		PORTB &= ~( 1 << PB4);
		_delay_ms(2);
}
void close_hole()
{
		PORTB |= ( 1 << PB4);
		PORTB &= ~( 1 << PB3);
		_delay_ms(600);
		PORTB &= ~( 1 << PB3);
		PORTB &= ~( 1 << PB4);
		_delay_ms(3);
}

unsigned char button_state()
{
	if (!(PINB & (1 << PB2)))
	{
		_delay_ms(30);
		if (!(PINB & (1 << PB2))) return 1;
	}
	return 0;
}

unsigned char stopped()
{
	if (!(PIND & (1 << PD7)))
	{
		_delay_ms(30);
		if (!(PIND7 & (1 << PD7))) return 1;
	}
	return 0;
}

void feed_dog(void) 
{
	init_pump_pins();

	// analog read
	ADC_init();

	// Activate pull res
	PORTA |= (1 << PA0);
	PORTB |= (1 << PB7);

	// led off = working
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
	
	// setup btn
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	while (1) {
		if (button_state()) {
			PORTD ^= (1 << PD7);
			_delay_ms(300);			
		}
		if (stopped()) continue;
		
		if (ADC_get(PA0_adc) <= WATER_LEVEL_LOW) {
			/* Turn on water pump */
			PORTB |= ( 1 << PB5);
			PORTB &= ~( 1 << PB6);
			// wait for filling
			while (ADC_get(PA0_adc) <= WATER_LEVEL_LOW);
			/* Turn off water pomp */
			PORTB &= ~( 1 << PB5);
			PORTB &= ~( 1 << PB6);
		} 

		// No food
		if ((PINB & (1 << PB7)) == 0) {
			open_hole();
			// wait for filling
			while ((PINB & (1 << PB7)) == 0);
			close_hole();
		} 

	}
}


int main(void) {
	feed_dog();
}


